<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 30/06/14
 * Time: 14:42
 */

namespace Controller;

use Service\Calendar\TaskCalendar;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Util\Messages;

class Calendar extends \Generated\Controller\Calendar{

    /** @var \Service\Calendar  */
    protected $calendarService;
    /**
     * @var Utilities
     */
    private $environment;
    /**
     * @var \Service\Project
     */
    private $projectService;

    public function __construct(Manager $dataSourceManager,\Service\Calendar $calendarService,\Service\Project $projectService,Utilities $environment) {
        parent::__construct($dataSourceManager);
        $this->calendarService = $calendarService;
        $this->environment = $environment;
        $this->projectService = $projectService;
    }

    public function findDefaultCalendar(Request\Filter $request, Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->environment->getCurrentOrganizationId());
        $request->getFilterCriteria()->addCondition("TYPE", "=", 'D');

        parent::find($request, $response);
    }
    public function findBasicCalendar(Request\Filter $request, Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->environment->getCurrentOrganizationId());

        parent::find($request, $response);
    }
    public function defaultCalendarDelete(Request\DataSet $request, Response $response){
        $rows = $request->getDataSet()->getRows();
        try {
            foreach ($rows as $row){
                $this->calendarService->defaultCalendarDelete($row['ID']);
            }
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        } catch (\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_POSSIBLE_TO_DELETE_CALENDAR_ALREADY_USED,
                Response\Message::TYPE_WARNING));
        }
    }

    public function defaultCalendarSave(Request\DataSet $request, Response $response)
    {
        $row         = $request->getDataSet()->getRows()[0];
        $id          = $row["ID"];
        $name        = $row["NAME"];
        $monthlyDays = isset($row["MONTHLY_DAYS"]) ? $row["MONTHLY_DAYS"] : null;
        $weeklyHours = isset($row["WEEKLY_HOURS"]) ? $row["WEEKLY_HOURS"] : null;
        $status      = $row["STATUS"];
        $calendar    = $this->calendarService->saveEditCalendar($id,$name,$monthlyDays,$weeklyHours,$status,\Service\Calendar::CALENDAR_DEFAULT);
        $response->addDataset(new DataSet('defaultCalendar', array('ID' => $calendar->getId())));
        $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
    }

    public function delete(Request $request, Response $response, $showMessage = true)
    {
        parent::delete($request, $response);
        $this->environment->treatResponseError($response, 'Não foi possível excluir a categoria selecionada. Parece que ela
        está sendo usada em algum outro lugar.');
    }
    public function findCalendarDates(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $month = Utilities::getValueInFilterDataSourceByName($filters,'MONTH');
        $year = Utilities::getValueInFilterDataSourceByName($filters,'YEAR');
        $calendarId = Utilities::getValueInFilterDataSourceByName($filters,'CALENDAR');

        $dataSet = $this->calendarService->findExceptionCalendarDates($calendarId,$month,$year);
        $response->addDataset(new DataSet('CalendarDetail', $dataSet));
    }

    public function getCalendarModel(Request\Filter $request,Response $response){

        $response->addDataset(new DataSet('CalendarModel', $this->calendarService->getCalendarModel()));
    }

    public function bringTaskCalendarHourByAllocation(Request\DataSet $request, Response $response){

        $param = $request->getDataSet()->getRows();
        $idAllocation = $param[0]['ALLOCATION'];
        $hourResource = $this->calendarService->getCalendarByAllocation($idAllocation);
        $hourProject = $this->calendarService-> getCalendarProjectByAllocation($idAllocation);
        if($hourProject>$hourResource){
            $response->addDataSet(new DataSet('bringTaskCalendarHourByAllocation', array($hourProject)));
        }else{
            $response->addDataSet(new DataSet('bringTaskCalendarHourByAllocation', array($hourResource)));
        }

    }

    public function projectCalendar(Request\Filter $request, Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = $param[0]['value'];
        $calendar = $this->projectService->getDataSetForEditProjectCalendar($projectId);
        $response->addDataSet(new DataSet('projectCalendar', $calendar ));
    }
    public function saveProjectCalendar(Request\DataSet $request, Response $response){
        try{
            $param = $request->getDataSet()->getRows();
            $param = $param[0];
            list($hourDay, $monthlyDays, $name, $weeklyHours, $days, $calendarId) = $this->ValidateParamsCalendar($param);
            $projectId = $param['PROJECT_ID'];
            $this->calendarService->saveProjectCalendar($projectId,$calendarId,$hourDay,$monthlyDays,$name,$weeklyHours,$days);

        }catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(),Response\Message::TYPE_WARNING));
        }
    }
        //odd calendar
    protected function checkBoxTrueEqualA($value){
        if($value == "true" || $value == "A"){
            $value = "A";
        }else{
            $value = "I";
        }
        return $value;
    }

    public function saveEditCalendar(Request\DataSet $request,Response $response){
        try {
            $param = $request->getDataSet()->getRows();
            $param = $param[0];
            $idEntity = $param['ID'];
            $name = $param['NAME'];
            $status = $this->checkBoxTrueEqualA($param['STATUS']);
            $beginWeek = $param['BEGIN_WEEK'];
            $endWeek = $param['END_WEEK'];
            $hour = $param['HOUR'];
            $beginHour = new \DateTime($param['BEGIN_HOUR']);
            $endHour = new \DateTime($param['END_HOUR']);
            $weeklyHours = $param['WEEKLY_HOURS'];
            $monthlyDays = $param['MONTHLY_DAYS'];
            $businessPartner = $param['BUSINESS_PARTNER'];
            $asset = empty($param['ASSET']) ? null:  $param['ASSET'];
            $cost = empty($param['COST']) ? null: $param['COST'];
            $entity = empty($param['ENTITY']) ? null: $param['ENTITY'];
            $this->calendarService->saveEditCalendar($idEntity, $name, $status, $beginWeek, $endWeek, $hour, $beginHour, $endHour,
                $weeklyHours, $monthlyDays, $businessPartner, $asset, $cost, $entity);
            if ($entity == 'Project') {
                // $this->projectService->recalculateAllDatesOfTheProjectTasks($idEntity);
            }
            $response->addMessage(new Response\Message('Calendario alterado'));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro : '.$e->getMessage(),Response\Message::TYPE_WARNING));
        }
    }
    /**
     * @param $param
     * @return array
     */
    public function ValidateParamsCalendar($param)
    {

        $hourDay = $param['HOUR']*3600;
        $monthlyDays = $param['MONTHLY_DAYS'];
        $name = $param['NAME'];
        $weeklyHours = $param['WEEKLY_HOURS'];
        $days = '';
        foreach (TaskCalendar::$ARRAY_WEEK as $week => $value) {
            if (!empty($param[$week])) {
                if ($param[$week] == 'Y') {
                    $days .= $week . '|';
                }
            }
        }
        $calendarId = empty($param['ID']) ? null : $param['ID'];
        return array($hourDay, $monthlyDays, $name, $weeklyHours, $days, $calendarId);
    }

} 