<?php
namespace Controller;

use Util\Utilities;
use Util\Messages;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\File;
use Zeedhi\Framework\Routing\Exception;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Meeting {

    protected $meetingService;

    public function __construct(\Service\Meeting $meetingService) {
        $this->meetingService = $meetingService;
    }

    public function createMeetingPlanning(Request\DataSet $request,Response $response){
        ini_set('max_execution_time','600');
        set_time_limit(600);
        $rows  = $request->getDataSet()->getRows();
        try {
            foreach($rows as $row){

                list($name,$recurrenceType, $id, $status, $beginDate, $occurrences, $exceptionQuantitative, $exceptionDataEnd,
                    $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly,
                    $weekDayException, $orderDay, $weekDay, $yearDay, $workTime,$beginTime)
                    = Project::builderExceptionRow($row);
                $this->meetingService->createRecurrenceMeetingPlanning($name,$recurrenceType, $id, $status, $beginDate, $occurrences,
                    $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat,
                    $yearDayException, $monthlyDay, $monthly, $weekDayException, $orderDay, $weekDay, $yearDay, $workTime, $beginTime);

            }
            $response->addNotification(new Response\Notification('Ocorrência de reunião criada com sucesso'));
        } catch (\Exception $e) {
            if($e->getCode()==33){
                $response->setError(new Response\Error($e->getMessage(). ' Code:' .$e->getCode(), $e->getCode()));
            }
            $response->setError(new Response\Error(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION .$e->getMessage(). ' Code:' .$e->getCode(), $e->getCode()));
        }

    }

    public function saveMeeting(Request\DataSet $request,Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        $dataSet = array();
        try {
            foreach ($rows as $row) {

                list($id, $name, $description, $managerId, $resourceIds, $projectTypeId, $clientIds, $productIds,
                    $projectSprintIds, $taskStatusIds, $taskLabelsIds, $taskPeriodEnd, $taskPeriodStart, $issueStatusIds,
                    $issueLabelsIds, $issuePeriodEnd, $issuePeriodStart, $beginDate, $time, $projectStatusId, $duration, $allowScheduleFilter) = $this->buildRowMeeting($row);

                $meetingProject = $this->meetingService->saveMeeting($id, $name, $description, $managerId, $resourceIds, $projectTypeId, $clientIds, $productIds,
                    $projectSprintIds, $taskStatusIds, $taskLabelsIds, $taskPeriodEnd, $taskPeriodStart, $issueStatusIds,
                    $issueLabelsIds, $issuePeriodEnd, $issuePeriodStart, $beginDate, $time, $projectStatusId, $duration, $allowScheduleFilter);
                $dataSet[] = $meetingProject->getId();
            }
            $response->addNotification(new Response\Notification('Reunião salva com sucesso'));
            $response->addDataset(new DataSet('projects', $dataSet));
        } catch (\Exception $e) {
            throw new \Exception(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage() . ' Code:' .$e->getCode());
        }
    }

    public function removeMeeting(Request\Row $request,Response $response){
        $row  = $request->getRow();
        try {
            $meetingId = $row["ID"];
            $this->meetingService->removeMeeting($meetingId);
            $response->addNotification(new Response\Notification('Reunião removida com sucesso'));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeSchedule(Request\Row $request,Response $response){
        $row  = $request->getRow();
        try {
            $meetingId = $row["TASK_ID"];
            $reload = $this->meetingService->removeSchedule($meetingId);
            $response->addDataSet(new DataSet('reload', array('RELOAD' =>$reload)));
            $response->addNotification(new Response\Notification('Registro removido com sucesso'));
        } catch (\Exception $e) {
            $response->setError(new Response\Error(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage().' Code:' .$e->getCode(), $e->getCode()));
        }
    }

    public function findAllMeeting(Request\Filter $request,Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $dataSet = $this->meetingService->findAllMeeting($conditions);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findInfoMeeting(Request\Filter $request,Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $project = Utilities::getValueInFilterDataSourceByName($filter,'ID');

            $dataBasic = $this->meetingService->findInfoMeeting($project);

            $response->addDataset(new DataSet("Basic", $dataBasic ));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findSprints(Request\Filter $request, Response $response) {
        try {
            $param = $request->getFilterCriteria()->getConditions();
            $projectIds = Utilities::getValueInFilterDataSourceByName($param,'PROJECTS');
            if(empty($projectIds)){
                $products = $this->meetingService->findSprints();
            }else{
                $products = $this->meetingService->findSprints($projectIds);
            }
            $response->addDataSet(new DataSet('products', $products));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    

    public function getFollowerInMeeting(Request\Filter $request, Response $response)
    {
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $projectId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT');
            $followers = $this->meetingService->getFollowerInMeeting($projectId);

            $response->addDataSet(new DataSet('followers', $followers));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    public function getNextTaskOrderByTypeAndSchedule(Request\Filter $request, Response $response)
    {
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $type = Utilities::getValueInFilterDataSourceByName($param, 'TYPE');
            $taskId = Utilities::getValueInFilterDataSourceByName($param, 'TASK');
            $followers = $this->meetingService->getNextTaskOrderByTypeAndSchedule($taskId,$type);

            $response->addDataSet(new DataSet('getNextTaskOrderByTypeAndSchedule', $followers));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findSchedule(Request\Filter $request, Response $response)
    {
        try {
            $param = $request->getFilterCriteria()->getConditions();
            $taskId = Utilities::getValueInFilterDataSourceByName($param, 'TASK');
            $meetingId = Utilities::getValueInFilterDataSourceByName($param, 'MEETING');
            $products = $this->meetingService->findSchedule($taskId,$meetingId);

            $response->addDataSet(new DataSet('products', $products));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findScheduleByMeeting(Request\Filter $request, Response $response)
    {
        try {
            $param = $request->getFilterCriteria()->getConditions();
            $meetingId = Utilities::getValueInFilterDataSourceByName($param, 'MEETING');
            $products = $this->meetingService->findScheduleByMeeting($meetingId);

            $response->addDataSet(new DataSet('products', $products));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveSchedule(Request\DataSet $request, Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as $row) {
                list($taskId, $classificationId, $description, $name, $parentId, $projectId, $resources, $taskStatus,
                    $isSchedule, $labels, $labelsNotRelatedUser, $labelsRelatedUser, $scheduleRelationshipIds, $meetingId,
                    $commitmentDate, $priority) =
                    $this->buildRowSchedule($row);
                list($products,$reload) = $this->meetingService->editSchedule($taskId, $projectId, $classificationId, $description, $name,
                    $parentId, $resources, $taskStatus, $isSchedule, $labelsNotRelatedUser, $labelsRelatedUser,
                    $scheduleRelationshipIds,$meetingId,$commitmentDate, $priority);
                $response->addDataSet(new DataSet('products', [$products]));
                $response->addDataSet(new DataSet('reload', array('RELOAD' =>$reload)));
            }
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    public function saveTaskOrder(Request\DataSet $request, Response $response)
    {
        try{
            $rows = $request->getDataSet()->getRows();
            $reload = false;
            foreach ($rows as $row) {
                list($taskId, $scheduleId, $taskOrder) = $this->buildRowSaveTaskOrder($row);
                $reloadTemp = $this->meetingService->saveTaskOrder($taskId, $scheduleId, $taskOrder);
                if($reloadTemp){
                    $reload = $reloadTemp;
                }
            }

            $response->addDataSet(new DataSet('reload', array('RELOAD' =>$reload)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveScheduleListOrder(Request\DataSet $request, Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $reload = false;
            foreach($rows as $row){
                list($id, $taskOrder) = $this->buildRowSaveScheduleListOrder($row);
                $reloadTemp = $this->meetingService->saveScheduleListOrder($id, $taskOrder);
                if($reloadTemp){
                    $reload = $reloadTemp;
                }
            }

            $response->addDataSet(new DataSet('reload', array('RELOAD' =>$reload)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findAttachmentMeeting(Request\Filter $request, Response $response){
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $id = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT');
            $resources = $this->meetingService->findAttachmentMeeting($id);
            $response->addDataSet(new DataSet('Attachment', $resources));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function taskListSchedule(Request\Filter $request, Response $response)
    {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $id      = Utilities::getValueInFilterDataSourceByName($filters, 'TASK', true);
            $dataSet = $this->meetingService->findTaskListSchedule($id, $filters);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function issueListSchedule(Request\Filter $request, Response $response)
    {
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $id      = Utilities::getValueInFilterDataSourceByName($filters, 'TASK', true);
            $dataSet = $this->meetingService->findIssueListSchedule($id, $filters, $page, $pageSize);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function findIssueListScheduleFromProject(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $meetingProjectId      = Utilities::getValueInFilterDataSourceByName($conditions, 'MEETING_PROJECT_ID', true);
            $scheduleId     = Utilities::getValueInFilterDataSourceByName($conditions, 'TASK', true);
            $projectId     = Utilities::getValueInFilterDataSourceByName($conditions, 'PROJECT', true);
            
            $dataSet = $this->meetingService->findIssueListScheduleFromProject($projectId, $scheduleId, $conditions, $page, $pageSize,$meetingProjectId);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveIssueListScheduleFromProject(Request\DataSet $request, Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        try {
            foreach ($rows as $row) {
                $scheduleId = $row['PAUTA_ID'];
                $labels = empty($row['LABELS']) ? array() : $row['LABELS'];
                $listIssueToSchedule = $row['LIST_ISSUE_TO_SCHEDULE'];
                $this->meetingService->saveIssueListScheduleFromProject($scheduleId, $listIssueToSchedule, $labels);
                $response->addNotification(new Response\Notification('Relacionamento salvo com sucesso'));
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    

    public function findProductMeeting(Request\Filter $request, Response $response)
    {
        try {
            $param = $request->getFilterCriteria()->getConditions();
            $projectId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT');
            $name = Utilities::getValueInFilterDataSourceByName($param, 'NAME');
            $products = $this->meetingService->findProductMeeting($projectId, $name);
            

            $response->addDataSet(new DataSet('products', $products));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function findProjectMeeting(Request\Filter $request, Response $response)
    {
        try {
            $param = $request->getFilterCriteria()->getConditions();
            $projectId = Utilities::getValueInFilterDataSourceByName($param, 'ID', true);
            $products = $this->meetingService->findProjectMeeting($projectId,$param);

            $response->addDataSet(new DataSet('products', $products));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param $row
     * @return array
     */
    protected function buildRowMeeting($row)
    {
        $id              = $row["ID"];
        $name            = $row["NAME"];
        $description     = $row["DESCRIPTION"];
        $managerId       = $row["MANAGER_ID"];
        $beginDate       = $row["BEGIN_DATE"];
        $time            = $row["TIME"];
        $resourceIds     = empty($row["RESOURCE"]) ? array() : $row["RESOURCE"];
        $projectTypeId   = empty($row["PROJECT_TYPE_ID"]) ? null : $row["PROJECT_TYPE_ID"];
        $projectStatusId = empty($row["PROJECT_STATUS_ID"]) ? null : $row["PROJECT_STATUS_ID"];
        $duration        = empty($row["TOTAL_TIME"]) ? 3600 : Utilities::timeToSeconds($row["TOTAL_TIME"]);
        $clientIds       = empty($row['CLIENT']) ? array() : $row['CLIENT'];
        $productIds       = empty($row['PRODUCT']) ? array() : $row['PRODUCT'];
        $projectSprintIds = empty($row['PROJECT_SPRINT']) ? array() : $row['PROJECT_SPRINT'];
        $allowScheduleFilter = $row['ALLOW_SCHEDULE_FILTER'];

        $taskStatusIds = empty($row['TASK_STATUS']) ? array() : $row['TASK_STATUS'];
        $taskLabelsIds = empty($row['TASK_LABELS']) ? array() : $row['TASK_LABELS'];
        
        if (isset($row['TASK_PERIOD'])) {
            $taskPeriodEnd = empty($row['TASK_PERIOD']['end']) ? null :
                \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $row['TASK_PERIOD']['end'] . ' 00:00:00');

            $taskPeriodStart = empty($row['TASK_PERIOD']['start']) ? null :
                \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $row['TASK_PERIOD']['start'] . ' 00:00:00');
        } else {
            $taskPeriodStart = null;
            $taskPeriodEnd = null;
        }
        
        if (isset($row['ISSUE_PERIOD'])) {
            $issuePeriodEnd = empty($row['ISSUE_PERIOD']['end']) ? null :
                \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $row['ISSUE_PERIOD']['end'] . ' 00:00:00');
    
            $issuePeriodStart = empty($row['ISSUE_PERIOD']['start']) ? null :
                \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $row['ISSUE_PERIOD']['start'] . ' 00:00:00');
        } else {
            $issuePeriodStart = null;
            $issuePeriodEnd = null;
        }

        $issueStatusIds = empty($row['ISSUE_STATUS']) ? array() : $row['ISSUE_STATUS'];
        $issueLabelsIds = empty($row['ISSUE_LABELS']) ? array() : $row['ISSUE_LABELS'];
        

        return array($id, $name, $description, $managerId, $resourceIds, $projectTypeId, $clientIds, $productIds,
            $projectSprintIds, $taskStatusIds, $taskLabelsIds, $taskPeriodEnd, $taskPeriodStart, $issueStatusIds,
            $issueLabelsIds, $issuePeriodEnd, $issuePeriodStart, $beginDate, $time, $projectStatusId, $duration, $allowScheduleFilter);
    }

    /**
     * @param $row
     * @return array
     */
    protected function buildRowSchedule($row)
    {
        $taskId = $row['TASK_ID'];
        $priority       = empty($row["PRIORITY"]) ? null : $row["PRIORITY"];
        $projectId = $row['PROJECT_ID'];
        $classification = empty($row["CLASSIFICATION"]) ? null : $row["CLASSIFICATION"];
        $description = empty($row["DESCRIPTION"]) ? null : $row["DESCRIPTION"];
        $name = empty($row["NAME"]) ? null : $row["NAME"];
        $parent = empty($row["PARENT"]) ? null : $row["PARENT"];
        $taskStatus = empty($row["TASK_STATUS"]) ? null : $row["TASK_STATUS"];
        $scheduleRelationshipIds = empty($row['SCHEDULE_RELATIONSHIP']) ? array() : $row['SCHEDULE_RELATIONSHIP'];
        $meetingId = empty($row['MEETING_ID']) ? null : $row['MEETING_ID'];
        $commitmentDate = empty($row['COMMITMENT_DATE']) ? null :\DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT,$row['COMMITMENT_DATE'].' 00:00:00');
        $resources = empty($row['RESOURCES']) ? array() : $row['RESOURCES'];
        $isSchedule = empty($row['IS_SCHEDULE']) ? 'N': $row['IS_SCHEDULE'];
        $labels = empty($row['LABELS']) ? array(): $row['LABELS'];
        $labelsNotRelatedUser = empty($row['LABELS_NOT_RELATED_USER']) ? array(): $row['LABELS_NOT_RELATED_USER'];
        $labelsRelatedUser = empty($row['LABELS_RELATED_USER']) ? array(): $row['LABELS_RELATED_USER'];
        return array($taskId, $classification, $description, $name, $parent, $projectId, $resources,$taskStatus,
            $isSchedule, $labels, $labelsNotRelatedUser, $labelsRelatedUser, $scheduleRelationshipIds,$meetingId,
            $commitmentDate, $priority);
    }

    /**
     * @param $row
     * @return array
     */
    protected function buildRowSaveTaskOrder($row)
    {
        $taskId = $row['ID'];
        $scheduleId = $row['SCHEDULE_ID'];
        $taskOrder = $row['TASK_ORDER'];

        return array($taskId, $scheduleId, $taskOrder);
    }

    protected function buildRowSaveScheduleListOrder($row){
        $scheduleId = $row['ID'];
        $taskOrder = $row['TASK_ORDER'];

        return array($scheduleId, $taskOrder);
    }


    public function labelsByMeeting(Request\Filter $request, Response $response)
    {
        $param = $request->getFilterCriteria()->getConditions();
        $meetingId = Utilities::getValueInFilterDataSourceByName($param, 'MEETING');
        $typeTaskLabel = Utilities::getValueInFilterDataSourceByName($param, 'TYPE');
        $labels = $this->meetingService->findLabelsByMeeting($meetingId,$typeTaskLabel);

        $response->addDataSet(new DataSet('labelsByMeeting', $labels));
    }

    public function findRecordsOfMeeting(Request\Filter $request, Response $response)
    {

        $filters = $request->getFilterCriteria()->getConditions();
        $scheduleId = Utilities::getValueInFilterDataSourceByName($filters,'TASK',true);
        $projectId = Utilities::getValueInFilterDataSourceByName($filters,'PROJECT',true);
        $comments = $this->meetingService->findRecordsOfMeeting($scheduleId,$projectId,$filters);
        $response->addDataSet(new DataSet('comments', $comments));
    }

    public function findScheduleMeetingPlanning(Request\Filter $request, Response $response)
    {
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $projectId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT',true);
            $dataSet = $this->meetingService->findScheduleMeetingPlanning($projectId, $param, $page, $pageSize);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new Response\Message($ex->getMessage(), 'Error'));
        }
    }

    public function findTaskMeetingPlannig(Request\Filter $request, Response $response)
    {
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $projectId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT',true);
            $dataSet = $this->meetingService->findTaskMeetingPlannig($projectId, $param, $page, $pageSize);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new Response\Message($ex->getMessage(), 'Error'));
        }
    }

    public function findIssueMeetingPlannig(Request\Filter $request, Response $response)
    {
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $projectId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT',true);
            $dataSet = $this->meetingService->findIssueMeetingPlannig($projectId, $param, $page, $pageSize);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new Response\Message($ex->getMessage(), 'Error'));
        }
    }
} 