<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Messages;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\Framework\Controller\Exception;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;

class Structure
{
    protected $entityManager;
    protected $environment;
    protected $structureFactory;
    /** @var Permission  */
    protected $permissionService;
    /**
     * @var Repositories\Structure
     */
    public $structureRepository;

    protected $resourceRepository;
    /**
     * @var Calendar
     */
    private $serviceCalendar;

    public function __construct(){

        $this->environment = InstanceProvider::getEnvironment();
        $this->entityManager = InstanceProvider::getEntityManager();
        //$this->structureFactory = $structureFactory;
        //$this->permissionService = $permissionService;
        //$this->serviceCalendar = $serviceCalendar;
        //$this->structureRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Structure');
        //$this->resourceRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Resource');
        //$this->permissionRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Permission');
        \Util\InstanceProvider::getLog()->setLog('instanciando Structuce service',false,2);
    }

    public function createStructure($param){
        $entity = InstanceProvider::getFactory()->getStructure()->create($param['STRUCTURE_TYPE'], null, $param['BUSINESS_PARTNER'], $param['NAME'], $param['STATUS']);
        $this->entityManager->persist($entity);
        $this->entityManager->flush();
        return $entity;
    }

    public function getStructureForResource(Entities\Resource $resource){
        /** @var Entities\Permission[] $permissions */
        $permissions =  InstanceProvider::getRepository()->getPermission()->findBy(array('resource' =>$resource->getId()));
        foreach($permissions as $permission){
            $type = $permission->getType();
            if(is_null($type) || $type == Permission::IS_RESOURCE){
                return $permission->getStructure();
            }
        }
        return null;
    }

    public function getStructureIdsForManager(Entities\Resource $resource){
        /** @var Entities\Permission[] $permissions */
        $permissions =  InstanceProvider::getRepository()->getPermission()->findBy(array('resource' =>$resource->getId(),'type'=> Permission::IS_MANAGER));
        $structureIds = array();
        foreach($permissions as $permission){
            $structureIds[] = $permission->getStructure()->getId();
        }
        return $structureIds;
    }

    public function getStructureIdForResource(Entities\Resource $resource){
        $structure = $this->getStructureForResource($resource);
        if($structure){
            $structureId = $structure->getId();
        }else{
            $structureId = null;
        }
        return $structureId;
    }

    public function getStructureOrganization(){

        return InstanceProvider::getRepository()->getStructure()->findOneBy(array('parent' => null));
    }
    public function setUserStructure(array $structures,array $structureResourceIds = array()){
        if(!empty($structureResourceIds)){
            foreach ($structures as &$structure) {
                $structure['USER_STRUCTURE'] = (in_array($structure['ID'],$structureResourceIds));
            }
        }
        return $structures;
    }

    public function getStructureFatherBeforeOrganization(Entities\Structure $structure,Entities\Structure $organizationStructure = null){
        if(empty($organizationStructure)){
            $organizationStructure = $this->getStructureOrganization();
        }
        if($structure->getParent()){
            if($structure->getParent()->getParent()){
                $structureBeforeOrganization = $this->getStructureFatherBeforeOrganization($structure->getParent(),$organizationStructure);
            }else{
                $structureBeforeOrganization = $structure;
            }
        }else{
            $structureBeforeOrganization = $structure;
        }
        return $structureBeforeOrganization;
    }

    /**
     * @param $projectManager
     * @return array
     */
    public function findGroupManager($projectManager = null, $filterString = '') {
        $organization = $this->environment->getCurrentOrganizationId();
        $structures = '';
        if($projectManager){
            $resource = InstanceProvider::getRepository()->getResource()->find($projectManager);
            $structure = $this->getStructureForResource($resource);
            if($structure){
                $structures = $this->getStructuresAllIdsStructureChildes([$structure->getId()]);
                $structures = implode(',',$structures);
            }
        }
        $structures = InstanceProvider::getRepository()->getStructure()->getGroupManager($organization, $structures, $filterString);
        $structures = Utilities::fixParentsForTreeGrid($structures);
        return $structures;
    }

    /**
     * @param null $projectManagers
     * @return array
     */
    public function findGroupManagerInEditProject($projectManagers = null, $filter = '') {
        $organization = $this->environment->getCurrentOrganizationId();
        $structures = array();
        $structureIds = array();
        $structureResourceIds = array();
        $filterStructures = null;
        $message =  \Util\Messages::ERR_RESOURCE_WITHOUT_STRUCTURE_I;
        $isValid = true;
        if($projectManagers) {
            /** @var Entities\Structure[] $structures */
            $resourceRepository = InstanceProvider::getRepository()->getResource();
            foreach ($projectManagers as $projectManager) {
                $resource = $resourceRepository->find($projectManager);
                $structureResource = $this->getStructureForResource($resource);
                if(!$structureResource){
                    $isValid = false;
                    $message = $message . $resource->getName() . "<br/>";
                }else{
                    $structureFather = $this->getStructureFatherBeforeOrganization($structureResource);
                    $structureId = $structureFather->getId();
                    if (empty($structureIds)) {
                        $structureIds[] = $structureId;
                    } else if (!in_array($structureId, $structureIds)) {
                        $structureIds = array('0');
                    }
                    $structureResourceIds[] = $structureResource->getId();  
                }
                
            }
            if(!$isValid){
                 $message = $message . \Util\Messages::ERR_RESOURCE_WITHOUT_STRUCTURE_II;
                 throw new Exception($message);
            }
        }


                $filterString = Utilities::getFilterTableColumn($filter, 'NAME');
                $filterString = Utilities::createQueryFiltered($filterString);
                $structures = InstanceProvider::getRepository()->getStructure()->getGroupManager($organization, $filterStructures, $filterString);
                $structures = $this->setUserStructure($structures,$structureResourceIds);
                $structures = Utilities::fixParentsForTreeGrid($structures);
                return $structures;
    }

    /**
     * @param $structureId
     * @return array
     */
    public function findBasicCalendar($structureId) {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure $structure */
        $structure = InstanceProvider::getRepository()->getStructure()->find($structureId);
        if($structure && $structure->getCalendar()){
            $calendars = InstanceProvider::getService()->getCalendar()->repositoryCalendar->getCalendar($structure->getCalendar()->getId());
            foreach($calendars as &$calendar){
                $calendar['STRUCTURE'] = $structureId;
            }
        }else{
            $name = $this->environment->getNextEntityWithCalendar($structure)->getName();
            $calendars = array(array('PARENT_CALENDAR_TEXT'=>Messages::ALE_CALENDAR_IS_NOT_OWN.$name,
                                'CALENDAR_DEFAULT'=> "",
                                'ID'=> "",
                                'STRUCTURE'=> $structureId));
        }
        return $calendars;
    }
    public function saveEditCalendar($id,$calendarDefaultId,$structureId){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure $structure */
            $structure = InstanceProvider::getRepository()->getStructure()->find($structureId);
            $nameCalendar = $structure->getId().'_'.$structure->getName();
            $serviceCalendar = InstanceProvider::getService()->getCalendar();
            $calendar = $serviceCalendar->saveEditCalendar($id,$nameCalendar,null,null,'A',Calendar::CALENDAR_STRUCTURE);
            if($calendarDefaultId){
                $calendarDefault = $serviceCalendar->find($calendarDefaultId);
                $calendar->setCalendarDefault($calendarDefault);
                $structure->setCalendar($calendar);
                $this->entityManager->persist($calendar);
                $this->entityManager->persist($structure);
                $this->entityManager->flush();
                $nextStructureWithCalendar = null;
                if($structure->getParent()){
                    $nextStructureWithCalendar = $this->environment->getNextEntityWithCalendar($structure->getParent());
                }
                if($nextStructureWithCalendar){
                    $serviceCalendar->copyAllCalendarDetail($nextStructureWithCalendar->getCalendar(),$calendar);
                    $this->entityManager->flush();
                }
            }else{
                $structure->setCalendar(null);
                $serviceCalendar->calendarDelete($calendar);
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $calendar;
    }
    public function saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
        $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
        $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate)
    {
        $calendarDetail = InstanceProvider::getService()->getCalendar()->saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
            $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
            $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate);
        $structure = InstanceProvider::getRepository()->getStructure()->findOneBy(array('calendar'=>$calendar));
        $this->setAllWeeklyDaysInStructureChild($structure,$calendarDetail, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
            $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
            $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate);
        return $calendarDetail;

    }

    public function saveException($id,$recurrenceType,$name,$calendar,$type,$status,$beginDate,$occurrences,
                                    $exceptionQuantitative,$exceptionDataEnd,$endDate,$every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,
                                    $monthly,$weekDayException,$orderDay,$weekDay,$yearDay,$workTime){
        $calendarDetail = InstanceProvider::getService()->getCalendar()->saveException($id,$recurrenceType,$name,$calendar,$type,$status,$beginDate,$occurrences,
            $exceptionQuantitative,$exceptionDataEnd,$endDate,$every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,
            $monthly,$weekDayException,$orderDay,$weekDay,$yearDay,$workTime);
        $structure = InstanceProvider::getRepository()->getStructure()->findOneBy(array('calendar'=>$calendar));
        $this->setAllCalendarDetailInStructureChild($structure,$calendarDetail,$recurrenceType,$name,$type,$status,$beginDate,$occurrences,
            $exceptionQuantitative,$exceptionDataEnd,$endDate,$every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,
            $monthly,$weekDayException,$orderDay,$weekDay,$yearDay,$workTime);
        return $calendarDetail;

    }

    public function getStructures($filter = '') {
        $conditions = Utilities::createQueryFiltered($filter);
        return InstanceProvider::getRepository()->getStructure()->getStructuresByOrganization($this->environment->getCurrentOrganizationId(), $conditions);
    }

    public function findStructureByResponsible($responsibleIds) {
        $structures = array();
        if($responsibleIds){
            $structures = InstanceProvider::getRepository()->getStructure()->getStructuresOwnedByManager($responsibleIds);
            foreach ($structures as $structure) {
                $gotChildren = $this->getChildren($structure['ID']);
                foreach ($gotChildren as $gotChild) {
                    if(!in_array($gotChild, $structures)){
                        array_push($structures, $gotChild);
                    }
                }
            }
        }
        return InstanceProvider::getRepository()->getStructure()->getStructureByResponsible($this->environment->getCurrentOrganizationId(),$structures);
    }

    public function getStructure() {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Structure $structureRepository */
        $structureRepository = InstanceProvider::getRepository()->getStructure();
        return $structureRepository->getStructuresByOrganization($this->environment->getCurrentOrganizationId());
    }

    public function getStructuresParents($filter)
    {
        $structureId = $filter[0]['value'];
        $filterString = Utilities::getFilterTableColumn($filter, 'NAME');
        $filterString = Utilities::createQueryFiltered($filterString);
        $filterStringStatus = Utilities::getFilterTableColumn($filter, 'STATUS');
        if($filterStringStatus){
            $filterString .=' '. Utilities::createQueryFiltered($filterStringStatus);
        }
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Structure $structureRepository */
        $structureRepository = InstanceProvider::getRepository()->getStructure();
        $structures = $structureRepository->getStructuresNoParents($this->environment->getCurrentOrganizationId(), $structureId, $filterString);
        return Utilities::fixParentsForTreeGrid($structures, 'ID', 'PARENT');
    }

    public function getStructureParents($structureId) {
        return InstanceProvider::getRepository()->getStructure()->getStructureParents($this->environment->getCurrentOrganizationId(), $structureId);
    }

    public function getStructureTypes($filter) {
        $filterString = Utilities::createQueryFiltered($filter);
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\StructureType $structureTypeRepository */
        $structureTypeRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\StructureType');
        return $structureTypeRepository->getStructureTypes($this->environment->getCurrentOrganizationId(), $filterString);
    }

    public function getBusinessPartner() {
        $businessPartnerRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\BusinessPartner');
        return $businessPartnerRepository->getBusinessPartnerByOrganization($this->environment->getCurrentOrganizationId());
    }

    public function getStructureInformation($structure) {
        $structureRepository = InstanceProvider::getRepository()->getStructure();
        return $structureRepository->getStructureById($structure);
    }

    public function getEditStructure($structure)
    {
        $structureRepository = InstanceProvider::getRepository()->getStructure();
        $structure = $structureRepository->getEditStructureById($structure);
        if (!empty($structure[0]['BUSINESS_PARTNER'])) {
            $structure[0]['BUSINESS_PARTNER'] = explode(',', $structure[0]['BUSINESS_PARTNER']);
        }
        $parent = $structureRepository->findOneBy(array('parent' => $structure[0]['ID']));
        if ($parent) {
            $structure[0]['IS_PARENT'] = true;
        }
        return $structure;
    }

    public function getBusinessPartnerForEditStructure() {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $resourceRepository */
        $resourceRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        return $resourceRepository->getBusinessPartnerForEditStructure($this->environment->getCurrentOrganizationId());
    }
    public function saveStructure($structureId, $name, $structureType, $resource, $parent, $status,$timeSheetRequired = 'N' ,
                                  $replicateTimeSheetRequired = false){
        $structureModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure';
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();

            if(empty($parent)){
                $parent = $this->entityManager->getRepository($structureModel)->getOrganizationStructure($this->environment->getCurrentOrganizationId());
                $parent = $parent[0]['ID'];
            }
            $parent = $this->entityManager->find($structureModel, $parent);

            $structureType = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\StructureType', $structureType);
            if(!empty($structureId)){
                $structure = $this->editStructure($structureId, $name, $structureType, $resource, $parent, $status, $timeSheetRequired,
                    $replicateTimeSheetRequired);
            }else{
                $structure = $this->saveNewStructure($name, $structureType, $resource, $parent, $status, $timeSheetRequired);
            }
            $connection->commit();

            return array("ID" => $structure->getId());
        }catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function replicateTimeSheetRequiredInChildrenStructures(Entities\Structure $structure)
    {
        /** @var Entities\Structure[] $childrenStructures */
        $childrenStructures =InstanceProvider::getRepository()->getStructure()->findBy(array('parent' => $structure));
        foreach ($childrenStructures as $childrenStructure) {
            $childrenStructure->setTimeSheetRequired($structure->getTimeSheetRequired());
            $this->entityManager->persist($childrenStructure);
            $this->replicateTimeSheetRequiredInChildrenStructures($childrenStructure);
        }
    }

    /**
     * @param $replicateTimeSheetRequired
     * @param Entities\Structure $structure
     * @param $timeSheetRequired
     */
    public function setTimeSheetRequired($replicateTimeSheetRequired,Entities\Structure $structure,$timeSheetRequired)
    {
        if ($replicateTimeSheetRequired) {
            $structure->setTimeSheetRequired($timeSheetRequired);
            $this->replicateTimeSheetRequiredInChildrenStructures($structure);
            $this->entityManager->flush();
        }
    }

    public function editStructure($structureId, $name, $structureType, $resource,
                                  Entities\Structure $structureParent = null, $status, $timeSheetRequired = 'N',
                                  $replicateTimeSheetRequired = false)
    {

        /** @var Entities\Structure $structure */
        $structure = InstanceProvider::getRepository()->getStructure()->find($structureId);
        $newParent = false;
        if ($this->validateNewParent($structureParent, $structure)) {
            $newParent = true;
        }
        $structure = InstanceProvider::getFactory()->getStructure()->editStructure($structure, $name, $structureType, $status, $structureParent,
            $timeSheetRequired);
        if ($newParent) {
            InstanceProvider::getService()->getCalendar()->adjustCalendarToNewParent($structure->getCalendar(),
                $this->environment->getNextEntityWithCalendar($structure->getParent())->getCalendar());
        }
        $this->setTimeSheetRequired($replicateTimeSheetRequired, $structure,$timeSheetRequired);
        $this->entityManager->persist($structure);
        $this->entityManager->flush();

        $this->setManager($structureId, $resource);

        return $structure;
    }

    public function saveNewStructure($name, $structureType, $resource, $structureParent, $status, $timeSheetRequired = 'N') {

        $resourceModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource';

        $resourceRepository = $this->entityManager->getRepository($resourceModel);

        $structure = InstanceProvider::getFactory()->getStructure()->createStructure($name, $structureType, $status, $structureParent, $timeSheetRequired);

        $this->entityManager->persist($structure);
        $this->entityManager->flush();

        if(!is_array($resource)){
            $resource = array($resource);
        }
        $permissionService = InstanceProvider::getService()->getPermission();
        foreach ($resource as $eachResource) {
            if(!($eachResource instanceof $resourceModel)){
                $eachResource = $resourceRepository->find($eachResource);
            }
            $permissionService->createPermission($eachResource, $structure, 'M');
        }
        return $structure;
    }

    public function deleteStructure($structureId) {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure $structure */
        $structure = InstanceProvider::getRepository()->getStructure()->find($structureId);
        if(!empty($structure)) {
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Project $projectRepository*/

            $projectRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Project');
            $projects = $projectRepository->findProjectbyStructure($structureId);

            $resourceModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource';

            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $resourceRepository */
            $resourceRepository = $this->entityManager->getRepository($resourceModel);
            $resources = $resourceRepository->findResourceByStructureId($structureId);

            if(empty($projects) && empty($resources)) {
                $structure->setStatus('D');
                $structure->setStructureType(null);
                $this->entityManager->persist($structure);
                $this->deleteRecursiveStructure($structure);
                $this->entityManager->flush();
            } else {
                throw new Exception(\Util\Messages::ERR_NOT_SUCCESSFUL_STRUCTURE_DELETED);
            }

        }
    }

    public function getChildren($structureId) {
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Structure $structureRepository
         */
        $structureRepository = InstanceProvider::getRepository()->getStructure();
        $childrenStructures = $structureRepository->getChildren($this->environment->getCurrentOrganizationId(), $structureId);
        foreach ($childrenStructures as $childStructure) {
            $childrenStructures = array_merge($childrenStructures, $this->getChildren($childStructure['ID']));
        }
        return $childrenStructures;
    }

    public function getAllParentsById($structureId) {

        $structure =  InstanceProvider::getRepository()->getStructure()->find($structureId);
        $parentStructures = array();
        if($structure->getParent()){
            $parentId = $structure->getParent()->getId();
            $parentStructures[] = $structureId;
            $parentStructures = array_merge($parentStructures,$this->getAllParentsById($parentId));

        }
        return $parentStructures;
    }

    public function getStructuresFromManagerOnline($filter = ''){
        $filterString = Utilities::createQueryFiltered($filter);
        $children = array();
        $structures = $this->findGroupManager(null, $filterString);
        foreach ($structures as $structure) {
            $gotChildren = $this->getChildren($structure['ID']);
            foreach ($gotChildren as $gotChild) {
                if(!in_array($gotChild, $structures)){
                    array_push($children, $gotChild);
                }
            }
        }
        if(array_diff_key($children, $structures)){
              array_push($structures, array_diff_key($children, $structures));
        }
        return $structures;
    }

    public function getAllStructuresChildrens($structureIds){
        return InstanceProvider::getRepository()->getStructure()->getAllStructuresChildrens($structureIds);
    }

    public function getAllStructuresChildrensByManager($resourceIds){
        return InstanceProvider::getRepository()->getStructure()->getAllStructuresChildrensByManager($resourceIds);
    }

    public function  getStructuresOwnedByManagerOnline($managerIds = array(), $allManagers = false){
        if(empty($managerIds)) {
            array_push($managerIds, $this->environment->getCurrentUserId());
        }
        $structures = InstanceProvider::getRepository()->getStructure()->getStructuresOwnedByManager($managerIds, $allManagers);
        foreach ($structures as $structure) {
            $gotChildren = $this->getChildren($structure['ID']);
            foreach ($gotChildren as $gotChild) {
                if(!in_array($gotChild, $structures)){
                    array_push($structures, $gotChild);
                }
            }
        }

        return $structures;
    }

    /**
     * @param $structure
     */
    protected function deleteRecursiveStructure($structure)
    {
        /**@var Entities\Structure[] $structureParents */
        $structureParents = InstanceProvider::getRepository()->getStructure()->findBy(array('parent' => $structure));
        foreach ($structureParents as $structureParent) {
            $structureParent->setStatus('D');
            $this->entityManager->persist($structureParent);
            $this->deleteRecursiveStructure($structureParent);
        }
    }

    public function getFirstStructureId($resourceId){
        $organization = $this->environment->getCurrentOrganizationId();
        $structures = InstanceProvider::getRepository()->getStructure()->getFistStructureId($resourceId, $organization);
        return empty($structures) ? null : $structures[0];
    }

    public function getStructureByResource($resource) {
        /** @var Entities\Resource $resource */
        $resourceId = $resource->getId();
        $organizationId = $resource->getOrganization();
        $structureRepository = InstanceProvider::getRepository()->getStructure();
        $structures = $structureRepository->getStructureByResourceId($organizationId, $resourceId);

        $structureToReturn = empty($structures[0]['ID']) ? null : $structureRepository->find($structures[0]['ID']);

        return $structureToReturn;
    }

    public function findStructureWithNoResourceId(){
        $organizationId = $this->environment->getCurrentOrganizationId();
        return InstanceProvider::getRepository()->getStructure()->getStructuresByOrganization($organizationId);
    }

    public function setAllCalendarDetailInStructureChild($structure,Entities\CalendarDetail $calendarDetail,$recurrenceType,$name,
                  $type,$status,$beginDate,$occurrences,$exceptionQuantitative,$exceptionDataEnd,$endDate,
                  $every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,$monthly,$weekDayException,
                  $orderDay,$weekDay,$yearDay,$workTime)
    {   /** @var Entities\Structure[] $childStructures */
        $childStructures = InstanceProvider::getRepository()->getStructure()->findBy(array('parent'=>$structure));
        foreach($childStructures as $childStructure){
            $id = null;
            $calendar = $childStructure->getCalendar();
                if($calendar){
                    $this->saveExceptionByOrigin($calendarDetail, $recurrenceType, $name, $type, $status, $beginDate,
                        $occurrences, $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon,
                        $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $weekDayException,
                        $orderDay, $weekDay, $yearDay, $workTime, $calendar);

                }
            $this->setAllCalendarDetailInStructureChild($childStructure, $calendarDetail,$recurrenceType,$name,
                  $type,$status,$beginDate,$occurrences,$exceptionQuantitative,$exceptionDataEnd,$endDate,
                  $every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,$monthly,$weekDayException,
                  $orderDay,$weekDay,$yearDay,$workTime);
        }
    }
    public function setAllWeeklyDaysInStructureChild($structure,Entities\CalendarDetail $calendarDetail, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
            $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
            $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate)
    {   /** @var Entities\Structure[] $childStructures */
        $childStructures = InstanceProvider::getRepository()->getStructure()->findBy(array('parent'=>$structure));
        foreach($childStructures as $childStructure){
            $id = null;
            $calendar = $childStructure->getCalendar();
            if($calendar) {
                $this->saveWeeklyDaysByOrigin($calendarDetail, $name, $typeSunday, $dailyHoursSunday, $typeMonday,
                    $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                    $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate,
                    $endDate, $calendar);
            }
            $this->setAllWeeklyDaysInStructureChild($childStructure,$calendarDetail, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
                $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate);
        }
    }

    /**
     * @param Entities\CalendarDetail $calendarDetail
     * @param $recurrenceType
     * @param $name
     * @param $type
     * @param $status
     * @param $beginDate
     * @param $occurrences
     * @param $exceptionQuantitative
     * @param $exceptionDataEnd
     * @param $endDate
     * @param $every
     * @param $sun
     * @param $mon
     * @param $tue
     * @param $wed
     * @param $thu
     * @param $fri
     * @param $sat
     * @param $yearDayException
     * @param $monthlyDay
     * @param $monthly
     * @param $weekDayException
     * @param $orderDay
     * @param $weekDay
     * @param $yearDay
     * @param $workTime
     * @param $calendar
     */
    public function saveExceptionByOrigin(Entities\CalendarDetail $calendarDetail, $recurrenceType, $name, $type, $status, $beginDate, $occurrences, $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $weekDayException, $orderDay, $weekDay, $yearDay, $workTime, $calendar)
    {
        $calendarDetailChild = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'CalendarDetail')
            ->findOneBy(array('calendar' => $calendar->getId(), 'originCalendarDetail' => $calendarDetail->getId()));
        $id = null;
        if ($calendarDetailChild) {
            $id = $calendarDetailChild->getId();
        }
        $calendarDetailChild = InstanceProvider::getService()->getCalendar()->saveException($id, $recurrenceType, $name, $calendar, $type, $status, $beginDate, $occurrences,
            $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay,
            $monthly, $weekDayException, $orderDay, $weekDay, $yearDay, $workTime);
        $calendarDetailChild->setOriginCalendarDetail($calendarDetail);
        $this->entityManager->persist($calendarDetailChild);
        $this->entityManager->flush();
    }

    /**
     * @param Entities\CalendarDetail $calendarDetail
     * @param $name
     * @param $typeSunday
     * @param $dailyHoursSunday
     * @param $typeMonday
     * @param $dailyHoursMonday
     * @param $typeTuesday
     * @param $dailyHoursTuesday
     * @param $typeWednesday
     * @param $dailyHoursWednesday
     * @param $typeThursday
     * @param $dailyHoursThursday
     * @param $typeFriday
     * @param $dailyHoursFriday
     * @param $typeSaturday
     * @param $dailyHoursSaturday
     * @param $beginDate
     * @param $endDate
     * @param $calendar
     */
    public function saveWeeklyDaysByOrigin(Entities\CalendarDetail $calendarDetail, $name, $typeSunday,
                                           $dailyHoursSunday, $typeMonday, $dailyHoursMonday, $typeTuesday,
                                           $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                                           $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday,
                                           $dailyHoursSaturday, $beginDate, $endDate, $calendar)
    {
        $calendarDetailChild = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'CalendarDetail')
            ->findOneBy(array('calendar' => $calendar->getId(), 'originCalendarDetail' => $calendarDetail->getId()));
        $id = null;
        if ($calendarDetailChild) {
            $id = $calendarDetailChild->getId();
        }
        $calendarDetailChild = InstanceProvider::getService()->getCalendar()->saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday, $typeMonday,
            $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
            $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate);
        $calendarDetailChild->setOriginCalendarDetail($calendarDetail);
        $this->entityManager->persist($calendarDetailChild);
        $this->entityManager->flush();
    }
    public function findExceptionCalendarDatesStructure($structureId,$month,$year){
        return InstanceProvider::getService()->getCalendar()->findExceptionCalendarDatesStructure($structureId,$month,$year);
    }

    /**
     * @param $structureId
     * @param $resources
     */
    public function setManager($structureId, $resources)
    {
        $permissionService = InstanceProvider::getService()->getPermission();
        $permissionService->removeNotFoundPermission($structureId, $resources, Permission::IS_MANAGER);
        foreach ($resources as $eachResource) {
            $permission = $permissionService->permissionRepository->findOneBy(array('resource' => $eachResource, 'structure' => $structureId,
                'type' => Permission::IS_MANAGER));
            $permissionService->persistPermission($permission, $eachResource, $structureId, Permission::IS_MANAGER);
        }
    }
    public function getStructureByFilter($filterString){
        return InstanceProvider::getRepository()->getStructure()->getStructureByFilter($this->environment->getCurrentOrganizationId(),$filterString);
    }
    public function  getStructuresAllIdsStructureChildes(array $structures = array()){
        foreach ($structures as $structure) {
            $gotChildren = $this->getChildren($structure);
            foreach ($gotChildren as $gotChild) {
                if(!in_array($gotChild, $structures)){
                    array_push($structures, $gotChild['ID']);
                }
            }
        }
        array_unique($structures);
        return $structures;
    }
    public function  getStructuresAllIdsStructureParentByResourceIds(array $resourceIds = array()){
        $structures = array();
        foreach($resourceIds as $resourceId){
            $firstStructure = InstanceProvider::getRepository()->getStructure()->getStructureForResourceId($resourceId);
            if($firstStructure[0] && $firstStructure[0]['ID']){
                $structures[] = intval($firstStructure[0]['ID']);
            }else{
                throw new \Exception('Recurso Sem estrutura');
            }
        }
        $parentStructures = array();
        foreach ($structures as $structure) {
            $parentStructures = array_merge($parentStructures,$this->getAllParentsById($structure));
        }
        array_unique($parentStructures);
        return $parentStructures;
    }
    public function  getCurrentOrganizationStructure(){
        $structures = array();
       return ;
    }

    /**
     * @param Entities\Structure $structureParent
     * @param Entities\Structure $structure
     * @return bool
     */
    protected function validateNewParent(Entities\Structure $structureParent, $structure)
    {
        return ($structureParent && $structure->getParent() && $structure->getCalendar()
            && $structure->getParent()->getId() != $structureParent->getId()) || (!$structureParent && $structure->getParent());
    }
}
